"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const js_tokens_1 = __importDefault(require("js-tokens"));
const helper_validator_identifier_1 = require("@babel/helper-validator-identifier");
const kleur_1 = __importDefault(require("kleur"));
/**
 * Names that are always allowed as identifiers, but also appear as keywords
 * within certain syntactic productions.
 *
 * https://tc39.es/ecma262/#sec-keywords-and-reserved-words
 *
 * `target` has been omitted since it is very likely going to be a false
 * positive.
 */
const sometimesKeywords = new Set(["as", "async", "from", "get", "of", "set"]);
/**
 * Kleur styles for token types.
 */
function getDefs(kleur) {
    return {
        keyword: kleur.cyan,
        capitalized: kleur.yellow,
        jsxIdentifier: kleur.yellow,
        punctuator: kleur.yellow,
        number: kleur.magenta,
        string: kleur.green,
        regex: kleur.magenta,
        comment: kleur.grey,
        invalid: kleur.white().bgRed().bold,
    };
}
/**
 * RegExp to test for newlines in terminal.
 */
const NEWLINE = /\r\n|[\n\r\u2028\u2029]/;
/**
 * RegExp to test for the three types of brackets.
 */
const BRACKET = /^[()[\]{}]$/;
/**
 * Get the type of token, specifying punctuator type.
 */
const getTokenType = function (token) {
    if (token.type === "IdentifierName") {
        if ((0, helper_validator_identifier_1.isKeyword)(token.value) ||
            (0, helper_validator_identifier_1.isStrictReservedWord)(token.value, true) ||
            sometimesKeywords.has(token.value)) {
            return "keyword";
        }
        if (token.value[0] !== token.value[0].toLowerCase()) {
            return "capitalized";
        }
    }
    if (token.type === "Punctuator" && BRACKET.test(token.value)) {
        return "uncolored";
    }
    if (token.type === "Invalid" && token.value === "@") {
        return "punctuator";
    }
    switch (token.type) {
        case "NumericLiteral":
            return "number";
        case "StringLiteral":
        case "JSXString":
        case "NoSubstitutionTemplate":
            return "string";
        case "RegularExpressionLiteral":
            return "regex";
        case "Punctuator":
        case "JSXPunctuator":
            return "punctuator";
        case "MultiLineComment":
        case "SingleLineComment":
            return "comment";
        case "Invalid":
        case "JSXInvalid":
            return "invalid";
        case "JSXIdentifier":
            return "jsxIdentifier";
        default:
            return "uncolored";
    }
};
/**
 * Turn a string of JS into an array of objects.
 */
const tokenize = function* (text) {
    for (const token of (0, js_tokens_1.default)(text, { jsx: true })) {
        switch (token.type) {
            case "TemplateHead":
                yield { type: "string", value: token.value.slice(0, -2) };
                yield { type: "punctuator", value: "${" };
                break;
            case "TemplateMiddle":
                yield { type: "punctuator", value: "}" };
                yield { type: "string", value: token.value.slice(1, -2) };
                yield { type: "punctuator", value: "${" };
                break;
            case "TemplateTail":
                yield { type: "punctuator", value: "}" };
                yield { type: "string", value: token.value.slice(1) };
                break;
            default:
                yield {
                    type: getTokenType(token),
                    value: token.value,
                };
        }
    }
};
/**
 * Highlight `text` using the token definitions in `defs`.
 */
function highlightTokens(defs, text) {
    let highlighted = "";
    for (const { type, value } of tokenize(text)) {
        const colorize = defs[type];
        if (colorize) {
            highlighted += value
                .split(NEWLINE)
                .map((str) => colorize(str))
                .join("\n");
        }
        else {
            highlighted += value;
        }
    }
    return highlighted;
}
/**
 * Highlight `code`.
 */
function highlight(code, options = {}) {
    if (code === "") {
        return code;
    }
    const kleur = options.kleur ?? kleur_1.default;
    if (options.forceColor) {
        const kleurEnabledBefore = kleur.enabled;
        kleur.enabled = true;
        let result;
        try {
            result = highlightTokens(getDefs(kleur), code);
        }
        finally {
            kleur.enabled = kleurEnabledBefore;
        }
        return result;
    }
    else if (kleur.enabled) {
        return highlightTokens(getDefs(kleur), code);
    }
    else {
        return code;
    }
}
exports.default = highlight;
